@@ -34,6 +34,7 @@ from lib389.instance.options import General2Base, Slapd2Base, Backend2Base
 from lib389.paths import Paths
 from lib389.saslmap import SaslMappings
 from lib389.instance.remove import remove_ds_instance
+from lib389.index import Indexes
 from lib389.utils import (
     assert_c,
     is_a_dn,
@@ -308,6 +309,9 @@ class SetupDs(object):
 
             val = input('\nEnter the instance name [{}]: '.format(slapd['instance_name'])).rstrip()
             if val != "":
+                if len(val) > 80:
+                    print("Server identifier should not be longer than 80 symbols")
+                    continue
                 if not all(ord(c) < 128 for c in val):
                     print("Server identifier can not contain non ascii characters")
                     continue
@@ -536,7 +540,7 @@ class SetupDs(object):
         return True
 
     def _prepare_ds(self, general, slapd, backends):
-
+        self.log.info("Validate installation settings ...")
         assert_c(general['defaults'] is not None, "Configuration defaults in section [general] not found")
         self.log.debug("PASSED: using config settings %s" % general['defaults'])
         # Validate our arguments.
@@ -573,7 +577,7 @@ class SetupDs(object):
         assert_c(slapd['instance_name'] != 'admin', "Server identifier \"admin\" is reserved, please choose a different identifier")
 
         # Check that valid characters are used
-        safe = re.compile(r'^[#%:\w@_-]+$').search
+        safe = re.compile(r'^[:\w@_-]+$').search
         assert_c(bool(safe(slapd['instance_name'])), "Server identifier has invalid characters, please choose a different value")
 
         # Check if the instance exists or not.
@@ -600,7 +604,7 @@ class SetupDs(object):
 
         # Right now, the way that rootpw works on ns-slapd works, it force hashes the pw
         # see https://fedorahosted.org/389/ticket/48859
-        if not re.match('^\{[A-Z0-9]+\}.*$', slapd['root_password']):
+        if not re.match('^([A-Z0-9]+).*$', slapd['root_password']):
             # We need to hash it. Call pwdhash-bin.
             # slapd['root_password'] = password_hash(slapd['root_password'], prefix=slapd['prefix'])
             pass
@@ -649,9 +653,9 @@ class SetupDs(object):
         Actually does the setup. this is what you want to call as an api.
         """
 
-        self.log.debug("START: Starting installation...")
+        self.log.debug("START: Starting installation ...")
         if not self.verbose:
-            self.log.info("Starting installation...")
+            self.log.info("Starting installation ...")
 
         # Check we have privs to run
         self.log.debug("READY: Preparing installation for %s...", slapd['instance_name'])
@@ -670,17 +674,16 @@ class SetupDs(object):
                 self._install_ds(general, slapd, backends)
             except ValueError as e:
                 if DEBUGGING is False:
-                    self.log.fatal("Error: " + str(e) + ", removing incomplete installation...")
                     self._remove_failed_install(slapd['instance_name'])
                 else:
-                    self.log.fatal("Error: " + str(e) + ", preserving incomplete installation for analysis...")
-                raise ValueError("Instance creation failed!")
+                    self.log.fatal(f"Error: {str(e)}, preserving incomplete installation for analysis...")
+                raise ValueError(f"Instance creation failed!  {str(e)}")
 
             # Call the child api to do anything it needs.
             self._install(extra)
-        self.log.debug("FINISH: Completed installation for %s", slapd['instance_name'])
+        self.log.debug("FINISH: Completed installation for instance: slapd-%s", slapd['instance_name'])
         if not self.verbose:
-            self.log.info("Completed installation for %s", slapd['instance_name'])
+            self.log.info("Completed installation for instance: slapd-%s", slapd['instance_name'])
 
         return True
 
@@ -728,6 +731,15 @@ class SetupDs(object):
             for line in template_dse.readlines():
                 dse += line.replace('%', '{', 1).replace('%', '}', 1)
 
+        # Check if we are in a container, if so don't use /dev/shm for the db home dir
+        # as containers typically don't allocate enough space for dev/shm and we don't
+        # want to unexpectedly break the server after an upgrade
+        container_result = subprocess.run(["systemd-detect-virt", "-c"], stdout=subprocess.PIPE)
+        if container_result.returncode == 0:
+            # In a container, set the db_home_dir to the db path
+            self.log.debug("Container detected setting db home directory to db directory.")
+            slapd['db_home_dir'] = slapd['db_dir']
+
         with open(os.path.join(slapd['config_dir'], 'dse.ldif'), 'w') as file_dse:
             dse_fmt = dse.format(
                 schema_dir=slapd['schema_dir'],
@@ -745,18 +757,22 @@ class SetupDs(object):
                 rootdn=slapd['root_dn'],
                 instance_name=slapd['instance_name'],
                 ds_passwd=self._secure_password,  # We set our own password here, so we can connect and mod.
-                # This is because we never know the users input root password as they can validily give
+                # This is because we never know the users input root password as they can validly give
                 # us a *hashed* input.
                 ds_suffix=ds_suffix,
                 config_dir=slapd['config_dir'],
                 db_dir=slapd['db_dir'],
-                db_home_dir=slapd['db_home_dir']
+                db_home_dir=slapd['db_home_dir'],
+                ldapi_enabled="on",
+                ldapi=slapd['ldapi'],
+                ldapi_autobind="on",
             )
             file_dse.write(dse_fmt)
 
+        self.log.info("Create file system structures ...")
         # Create all the needed paths
         # we should only need to make bak_dir, cert_dir, config_dir, db_dir, ldif_dir, lock_dir, log_dir, run_dir?
-        for path in ('backup_dir', 'cert_dir', 'db_dir', 'ldif_dir', 'lock_dir', 'log_dir', 'run_dir'):
+        for path in ('backup_dir', 'cert_dir', 'db_dir', 'db_home_dir', 'ldif_dir', 'lock_dir', 'log_dir', 'run_dir'):
             self.log.debug("ACTION: creating %s", slapd[path])
             try:
                 os.umask(0o007)  # For parent dirs that get created -> sets 770 for perms
@@ -786,14 +802,14 @@ class SetupDs(object):
         # Copy in the collation
         srcfile = os.path.join(slapd['sysconf_dir'], 'dirsrv/config/slapd-collations.conf')
         dstfile = os.path.join(slapd['config_dir'], 'slapd-collations.conf')
-        shutil.copy2(srcfile, dstfile)
+        shutil.copy(srcfile, dstfile)
         os.chown(dstfile, slapd['user_uid'], slapd['group_gid'])
         os.chmod(dstfile, 0o440)
 
         # Copy in the certmap configuration
         srcfile = os.path.join(slapd['sysconf_dir'], 'dirsrv/config/certmap.conf')
         dstfile = os.path.join(slapd['config_dir'], 'certmap.conf')
-        shutil.copy2(srcfile, dstfile)
+        shutil.copy(srcfile, dstfile)
         os.chown(dstfile, slapd['user_uid'], slapd['group_gid'])
         os.chmod(dstfile, 0o440)
 
@@ -805,7 +821,9 @@ class SetupDs(object):
             args = ' '.join(ensure_list_str(result.args))
             stdout = ensure_str(result.stdout)
             stderr = ensure_str(result.stderr)
-            self.log.debug(f"CMD: {args} ; STDOUT: {stdout} ; STDERR: {stderr}")
+            # Systemd encodes some odd charecters into it's symlink output on newer versions which
+            # can trip up the logger.
+            self.log.debug(f"CMD: {args} ; STDOUT: {stdout} ; STDERR: {stderr}".encode("utf-8"))
 
             # Setup tmpfiles_d
             tmpfile_d = ds_paths.tmpfiles_d + "/dirsrv-" + slapd['instance_name'] + ".conf"
@@ -832,12 +850,24 @@ class SetupDs(object):
         if self.containerised:
             ds_instance.systemd_override = general['systemd']
 
+        # By default SUSE does something extremely silly - it creates a hostname
+        # that CANT be resolved by DNS. As a result this causes all installs to
+        # fail. We need to guarantee that we only connect to localhost here, as
+        # it's the only stable and guaranteed way to connect to the instance
+        # at this point.
+        #
+        # Use ldapi which would prevent the need
+        # to configure a temp root pw in the setup phase.
         args = {
+            SER_HOST: "localhost",
             SER_PORT: slapd['port'],
             SER_SERVERID_PROP: slapd['instance_name'],
             SER_ROOT_DN: slapd['root_dn'],
             SER_ROOT_PW: self._raw_secure_password,
-            SER_DEPLOYED_DIR: slapd['prefix']
+            SER_DEPLOYED_DIR: slapd['prefix'],
+            SER_LDAPI_ENABLED: 'on',
+            SER_LDAPI_SOCKET: slapd['ldapi'],
+            SER_LDAPI_AUTOBIND: 'on'
         }
 
         ds_instance.allocate(args)
@@ -850,6 +880,7 @@ class SetupDs(object):
             tlsdb.reinit()
 
         if slapd['self_sign_cert']:
+            self.log.info("Create self-signed certificate database ...")
             etc_dirsrv_path = os.path.join(slapd['sysconf_dir'], 'dirsrv/')
             ssca_path = os.path.join(etc_dirsrv_path, 'ssca/')
             ssca = NssSsl(dbpath=ssca_path)
@@ -867,7 +898,7 @@ class SetupDs(object):
                         tlsdb_inst = NssSsl(dbpath=os.path.join(etc_dirsrv_path, dir))
                         tlsdb_inst.import_rsa_crt(ca)
 
-            csr = tlsdb.create_rsa_key_and_csr()
+            csr = tlsdb.create_rsa_key_and_csr(alt_names=[general['full_machine_name']])
             (ca, crt) = ssca.rsa_ca_sign_csr(csr)
             tlsdb.import_rsa_crt(ca, crt)
             if general['selinux']:
@@ -876,8 +907,9 @@ class SetupDs(object):
 
         # Do selinux fixups
         if general['selinux']:
+            self.log.info("Perform SELinux labeling ...")
             selinux_paths = ('backup_dir', 'cert_dir', 'config_dir', 'db_dir',
-                             'ldif_dir', 'lock_dir', 'log_dir',
+                             'ldif_dir', 'lock_dir', 'log_dir', 'db_home_dir',
                              'run_dir', 'schema_dir', 'tmp_dir')
             for path in selinux_paths:
                 selinux_restorecon(slapd[path])
@@ -908,9 +940,23 @@ class SetupDs(object):
         if slapd['self_sign_cert']:
             ds_instance.config.set('nsslapd-security', 'on')
 
+        # Before we create any backends, create any extra default indexes that may be
+        # dynamically provisioned, rather than from template-dse.ldif. Looking at you
+        # entryUUID (requires rust enabled).
+        #
+        # Indexes defaults to default_index_dn
+        indexes = Indexes(ds_instance)
+        if ds_instance.ds_paths.rust_enabled:
+            indexes.create(properties={
+                'cn': 'entryUUID',
+                'nsSystemIndex': 'false',
+                'nsIndexType': ['eq', 'pres'],
+            })
+
         # Create the backends as listed
         # Load example data if needed.
         for backend in backends:
+            self.log.info(f"Create database backend: {backend['nsslapd-suffix']} ...")
             is_sample_entries_in_props = "sample_entries" in backend
             create_suffix_entry_in_props = backend.pop('create_suffix_entry', False)
             ds_instance.backends.create(properties=backend)
@@ -941,14 +987,6 @@ class SetupDs(object):
                     # Unsupported rdn
                     raise ValueError("Suffix RDN '{}' in '{}' is not supported.  Supported RDN's are: 'c', 'cn', 'dc', 'o', and 'ou'".format(suffix_rdn_attr, backend['nsslapd-suffix']))
 
-        # Initialise ldapi socket information. IPA expects this ....
-        ldapi_path = os.path.join(slapd['local_state_dir'], "run/slapd-%s.socket" % slapd['instance_name'])
-        ds_instance.config.set('nsslapd-ldapifilepath', ldapi_path)
-        ds_instance.config.set('nsslapd-ldapilisten', 'on')
-        ds_instance.config.set('nsslapd-ldapiautobind', 'on')
-        ds_instance.config.set('nsslapd-ldapimaprootdn', slapd['root_dn'])
-
-
         # Create all required sasl maps: if we have a single backend ...
         # our default maps are really really bad, and we should feel bad.
         # they basically only work with a single backend, and they'll break
@@ -973,6 +1011,7 @@ class SetupDs(object):
         else:
             self.log.debug("Skipping default SASL maps - no backend found!")
 
+        self.log.info("Perform post-installation tasks ...")
         # Change the root password finally
         ds_instance.config.set('nsslapd-rootpw', slapd['root_password'])
 
@@ -987,3 +1026,5 @@ class SetupDs(object):
         else:
             # Just stop the instance now.
             ds_instance.stop()
+
+        self.log.debug(" 🎉 Instance setup complete")
